#!/usr/bin/python

#############################################################################
## front end of the portfolio solver 'coach'                               ##
##                                                                         ##
##   version 1.0, March 2011                                               ##
##                                                                         ##
##   Yuri Malitsky, Ashish Sabharwal, Horst Samulowitz, Meinolf Sellmann   ##
##                                                                         ##
#############################################################################


import sys
import os
import getopt
import time

usage = "\nUSAGE: " + sys.argv[0] + " [--checkall] [-t,--timelimit TIMELIMIT] [-s,--seed RANDOMSEED]\n [-d,--tmpdir TMPDIR] [--nofilecleanup] [--testsolvers]\n [-h,--help] [-v,--verbose] BENCHNAME"

# set command-line argument defaults here
tmpdir         = "WORKING-DIR"
timelimit      = 50000000
seed           = "87360941"
nofilecleanup  = False
testsolvers    = False
verbosity      = 0
solution_line_length = 20     # how many variables to print per line when printing solution

timelimit_low_cutoff = 1300   # a bit higher than the 1200 cutoff for phase 1 of the competition
solver_default = 'lingeling'  # to be used, e.g., to convert preprocessed solution to original variables
timer_start    = 0
timer_prev     = 0

# set the name of the indicator file that coach-executables-test.sh
# creates to indicate that the test has been run at least once on the
# current machine
file_executables_test_indicator = "coach-executables-test-SUCCESS.txt"
prog_executables_test           = "coach-executables-test.sh"

# set main portfolio program, the 'intelligence' module
prog_portfolio       = "COACH/coach"
datafolder_portfolio = "COACH/DATA"

# set program names here
prog_timerun       = "utilities/timerun"
prog_features      = "./features"
file_features_test = "unif-k3-r4.2-v18000-c75600-S914323646-060.cnf"
prog_preprocessor  = "preprocessors/SatELite_v1.0_linux"
# paths to all SAT solvers
prog_solvers       = {}
#   solvers good for industrial/crafted instances
prog_solvers['clasp-1.3.6-x86-linux'] = 'solvers/clasp-1.3.6-x86-linux'
prog_solvers['cryptominisat']         = 'solvers/cryptominisat'
prog_solvers['glucose_static']        = 'solvers/glucose_static'
prog_solvers['lingeling']             = 'solvers/lingeling'
prog_solvers['LySATc']                = 'solvers/LySATc'
prog_solvers['LySATi']                = 'solvers/LySATi'
prog_solvers['march_hi']              = 'solvers/march_hi'
prog_solvers['march_nn']              = 'solvers/march_nn'
prog_solvers['minisat_static']        = 'solvers/minisat_static'
prog_solvers['mxc-sat09']             = 'solvers/mxc-sat09'
prog_solvers['precosat']              = 'solvers/precosat'
#   solvers good for random instances
prog_solvers['adaptg2wsat2009']       = 'solvers/adaptg2wsat2009'
prog_solvers['adaptg2wsat2009++']     = 'solvers/adaptg2wsat2009++'
prog_solvers['AshiFact']              = 'solvers/SATenstein'
prog_solvers['AshiCbmc']              = 'solvers/SATenstein'
prog_solvers['AshiR3fix']             = 'solvers/SATenstein'
prog_solvers['AshiHgen']              = 'solvers/SATenstein'
prog_solvers['AshiSwgcp']             = 'solvers/SATenstein'
prog_solvers['AshiQcp']               = 'solvers/SATenstein'
prog_solvers['gnovelty+2']            = 'solvers/gnovelty+2'
prog_solvers['gnovelty+2-H']          = 'solvers/gnovelty+2-H'
prog_solvers['hybridGM3']             = 'solvers/hybridGM3'
prog_solvers['kcnfs04SAT07']          = 'solvers/kcnfs04SAT07'
prog_solvers['picosat']               = 'solvers/picosat'
prog_solvers['saps']                  = 'solvers/ubcsatOrig'
prog_solvers['TNM']                   = 'solvers/TNM'

# Parameters of solvers --- initialize to empty string
param_solvers = {}
for name in prog_solvers:
  param_solvers[name] = ""

# Set parameters for SAPS
param_solvers['saps'] = " -alg saps -cutoff max -runs 100 -solve -r satcomp "

# Set parameters for various SATensteins
param_solvers['AshiFact']   = " -alg satenstein -varinfalse 1 -adaptive 0 -alpha 1.126 -clausepen 1 -performrandomwalk 0 -promisinglist 0 -ps 0.033 -scoringmeasure 3 -selectclause 1 -singleclause 0 -smoothingscheme 1 -tabusearch 0 -tiebreaking 1 -wp 0 -r satcomp -cutoff max -runs 100 -solve "
param_solvers['AshiCbmc']   = " -alg satenstein -varinfalse 1 -rwp 0.01 -singleclause 0 -randomwalk 1 -alpha 1.126 -performrandomwalk 1 -smoothingscheme 1 -rho 1 -clausepen 1 -selectclause 1 -promisinglist 0 -adaptive 1 -tiebreaking 1 -tabusearch 0 -scoringmeasure 1 -wp 0.01 -r satcomp  -cutoff max -runs 100 -solve "
param_solvers['AshiR3fix']   = " -alg satenstein -varinfalse 1 -singleclause 0 -alpha 1.126 -wp 0.04 -smoothingscheme 1 -performrandomwalk 0 -rho 0.17 -clausepen 1 -selectclause 1 -promisinglist 0 -adaptive 1 -tiebreaking 2 -tabusearch 0 -scoringmeasure 1 -r satcomp  -cutoff max -runs 100 -solve "
param_solvers['AshiHgen']   = " -alg satenstein -varinfalse 1 -adaptwalkprob 0 -theta 3 -singleclause 1 -randomwalk 3 -performrandomwalk 1 -selectclause 1 -promisinglist 0 -adaptiveprom 0 -heuristic 7 -adaptive 1 -phi 10 -tabusearch 0 -rdp 0.15 -r satcomp  -cutoff max -runs 100 -solve "
param_solvers['AshiSwgcp']   = " -alg satenstein -varinfalse 1 -singleclause 1 -randomwalk 3 -performrandomwalk 1 -heuristic 1 -clausepen 0 -selectclause 1 -promisinglist 0 -adaptive 0 -tabusearch 0 -rdp 0.01 -novnoise 0.1 -r satcomp  -cutoff max -runs 100 -solve "
param_solvers['AshiQcp']   = " -alg satenstein -varinfalse 1 -singleclause 1 -randomwalk 4 -performrandomwalk 1 -heuristic 5 -clausepen 0 -selectclause 1 -promisinglist 0 -adaptive 0 -rfp 0.07 -tabusearch 0 -scoringmeasure 1 -r satcomp  -cutoff max -runs 100 -solve "

# Parameters for the preprocessor
pre_instance_param_preprocessor = " +pre --verbosity=0 "

# Pre instance parameters of solvers --- initialize to a single space
pre_instance_param_solvers = {}
for name in prog_solvers:
  pre_instance_param_solvers[name] = " " 

# Set pre-instance parameters for MXCSAT
pre_instance_param_solvers['mxc-sat09'] = " -i "

# Set pre-instance parameters for SAPS
pre_instance_param_solvers['saps'] = " -i "

# Set pre-instance parameters for various SATensteins
pre_instance_param_solvers['AshiFact']     = " -i "
pre_instance_param_solvers['AshiCbmc']     = " -i "
pre_instance_param_solvers['AshiR3fix']    = " -i "
pre_instance_param_solvers['AshiHgen']     = " -i "
pre_instance_param_solvers['AshiSwgcp']    = " -i "
pre_instance_param_solvers['AshiQcp']      = " -i "


# names of solvers whose output is already "compliant" with the competition requirements
compliant_solvers = ['clasp-1.3.6-x86-linux', 'cryptominisat', 'glucose_static', 'lingeling', 'LySATi', 'march_hi', 'march_nn', 'mxc-sat09', 'precosat', 'AshiFact', 'AshiCbmc', 'AshiR3fix', 'AshiHgen', 'AshiSwgcp', 'AshiQcp', 'hybridGM3', 'gnovelty+2', 'gnovelty+2-H', 'saps', 'picosat', 'adaptg2wsat2009', 'adaptg2wsat2009++', 'TNM', 'kcnfs04SAT07']
# names of solvers whose output is "compliant" with minisat style
minisat_compliant_solvers = ['minisat_static', 'LySATc']

if len(prog_solvers) != len(compliant_solvers) + len(minisat_compliant_solvers):
  print "Warning: Number of solvers is not equal to the number of compliant or minisat-compliant solvers (" + str(len(prog_solvers)) + " vs " + str(len(compliant_solvers) + len(minisat_compliant_solvers)) + ")."

###################################
## main routine
###################################
def main():
  global tmpdir, timelimit, seed, verbosity
  global timer_start, timer_prev
  global prog_timerun, prog_features, prog_portfolio, prog_preprocessor, prog_solvers, param_solvers, pre_instance_param_solvers
  global prog_executables_test, file_executables_test_indicator

  # some variables and their defaults  
  run_preprocessor = False
  solvers_to_use  = []
  solvers_list = []
  for (solver, solver_path) in prog_solvers.items():
    solvers_list.append(solver)

  # print command line
  print "c COMMAND LINE: " + reduce(lambda x, y: x + ' ' + y, sys.argv)
  
  # parse command-line and obtain CNF filename
  filename_cnf = parse_command_line()
  for solver in ['AshiFact', 'AshiCbmc', 'AshiR3fix', 'AshiHgen', 'AshiSwgcp', 'AshiQcp', 'saps']:
    param_solvers[solver] += '-seed ' + seed + ' '
  print "c Parsed command line and set default values..."

  print "c Starting *WALL CLOCK* TIMER"
  timer_start = time.time()
  timer_prev  = timer_start

  if not os.path.exists(tmpdir):
    os.mkdir(tmpdir)

  # first check to make sure coach-executables-test.sh was executed at
  # least once on this system! Look for the "indicator" file it
  # generates
  file_executables_test_indicator = tmpdir + '/' + file_executables_test_indicator
  if not os.path.exists(file_executables_test_indicator):
    print "c"
    print "c ERROR: " + file_executables_test_indicator + " not found"
    if tmpdir != 'WORKING-DIR':
      test_commandline = "./" + prog_executables_test + ' ' + tmpdir
    else:
      test_commandline = "./" + prog_executables_test
    print "c"
    print "c Please first run \"" + test_commandline + "\""
    print "c   to make sure all necessary executables work properly."
    sys.exit(1)
  else:
    print "c Found " + file_executables_test_indicator
    print "c   indicating " + prog_executables_test + " was run successfully earlier"

  # now check to make sure all necessary executable and folders are present!
  checkAllFilesFoldersAvailable()
  printTimer()

  # Set results-file parameter for minisat_compliant_solvers
  filename_minisat_compliant_results = tmpdir + "/minisat_compliant_results.txt"
  for solver in minisat_compliant_solvers:
    param_solvers[solver] = ' ' + filename_minisat_compliant_results


  # if --testsolvers specified, run each solver for 5 seconds and quit
  if testsolvers:
    print "c TEST SOLVERS mode; will run each solver and preprocessor for 5 seconds."
    print "c   If a solver stops sooner than 5 seconds on a hard enough"
    print "c   test instance, then there is a problem!"
    filename_to_solve = filename_cnf
    runtime = "5"
    for solver in solvers_list:
      print "c Running " + solver + " for " + runtime + " seconds on " + filename_to_solve
      command = prog_timerun + ' ' + runtime + ' ' + prog_solvers[solver] + pre_instance_param_solvers[solver] + filename_to_solve + param_solvers[solver]
      print "c    " + command
      solverIn, solverOut, solverErr = os.popen3(command)
      solver_result = solverOut.read().rstrip()
      printTimer()
    print "c TEST COMPLETE"
    sys.exit(0)

  # compute features
  print "c Computing instance features..."
  command = prog_features + ' ' + filename_cnf
  print "c    " + command
  features_stream = os.popen(command)
  line = features_stream.readline()
  line = features_stream.readline().rstrip()
  features = line.replace(',', ' ')
  # If feeature computation fails, simply call default solver
  if len(features.split()) != 48:
    print "c ERROR: 48 features not computed correctly!"    
    print "c Calling default solver with no time limit!"
    command = prog_solvers[solver_default] + pre_instance_param_solvers[solver_default] + filename_cnf + param_solvers[solver_default]
    print "c    " + command
    # Execute solve command
    # Turn off pre-processing and simply call the default solver on 'filename_cnf'
    success = run_command_and_return_result(command, solver_default, False, filename_minisat_compliant_results, "", "")
    # Should never get here.
    if success == False:
      print "s UNKNOWN"
    # Exit and don't continue with the script
    sys.exit(0)

  print "c    Features found: " + features
  printTimer()

  # run portfolio solver to determine which individual solver to use
  command = prog_portfolio + ' ' + features + ' ' + datafolder_portfolio + "/ "
  if timelimit <= timelimit_low_cutoff:
    command += '1'
  else:
    command += '0'
  print "c Computing most promising solver to run..."
  print "c    " + command
  portfolio_stream = os.popen(command)
  for line in portfolio_stream:
    line = line.rstrip()
    words = line.split()
    if words[0] == 'PORTFOLIO-SUGGESTION':
      print "c    " + line
      if words[1] == "SOLVER":
        if len(words) != 5:
          print "c ERROR: invalid portfolio suggestion line; skpping it"
          continue
        solver_name = words[2]
        if (solver_name not in solvers_list):
          print "c ERROR: " + solver_name + " is not in the list of available solvers; skipping it"
          continue
        solver_time = words[3]
        if words[4] != '0':
          solver_pre = True
          run_preprocessor = True
        else:
          solver_pre = False
        solvers_to_use.append( (solver_name, solver_time, solver_pre) )
      else:
        print "c ERROR: invalid portfolio suggestion line; skpping it"
        continue
  printTimer()

  # declare the solver(s) to be used, with or without preprocessing
  if run_preprocessor:
    print "c Preprocessing ON (for solvers that need to use it)"
  else:
    print "c Preprocessing OFF"
  print "c Solvers and corresponding runtimes to be used, and whether on the preprocessed instance:"
  for (solver, runtime, usepre) in solvers_to_use:
    print "c   ", solver.ljust(15), '\t', runtime.rjust(15), '\t', usepre
  print "c Default solver to fall back on, in case everything fails:"
  print "c   ", solver_default.ljust(15), '\t', "no limit".rjust(15), '\t', False
  solvers_to_use.append( (solver_default, "-1", False) )

  # run the suggested SAT solvers and hope for the best!
  preprocessed_file_already_generated = False
  filename_cnf_pre = tmpdir + '/' + os.path.basename(filename_cnf) + ".pre"
  filename_varmap = filename_cnf_pre + ".varmap"

  # Set filename to solve to filename of the cnf
  filename_to_solve = filename_cnf
  
  # Make sure that files do not exist from previous runs
  if os.path.exists(filename_cnf_pre):
    os.remove(filename_cnf_pre)
  if os.path.exists(filename_varmap):
    os.remove(filename_varmap)
  
  success = False
  # Flag that indicates if Satelite has failed or not
  satelite_failed = False
  # Loop over solver schedule given by k-NN
  for (solver, runtime, usepre) in solvers_to_use:
    # If preprocessing flag is set to true, call Satelite
    if usepre:
      # Set filename_to_solve to the filename to be preprocessed
      filename_to_solve = filename_cnf_pre
      # do preprocessing if not already done or not already failed before
      if not preprocessed_file_already_generated and not satelite_failed:
        print "c <<<< Running SatElite to preprocess instance >>>>" 
        command = prog_preprocessor + pre_instance_param_preprocessor + filename_cnf + ' ' + filename_cnf_pre + ' ' + filename_varmap
        print "c    " + command
        # Execute Satelite and get return code
        return_code = os.system(command)
        print "c Satelite return code: " + str(return_code)
        # If the return code is not equal to 0, Satelite has failed
        if return_code != 0:
          print "c Satelite failed (Return code != 0). TURNING OFF preprocessing for all solvers."
          # Turn off pre-processing flag
          usepre = False
          # Set unpre-processed filename
          filename_to_solve = filename_cnf
          # Never call Satelite again on this instance
          satelite_failed = True
          # Make sure that temporary files are removed if exist
          if os.path.exists(filename_cnf_pre):
            os.remove(filename_cnf_pre)
          if os.path.exists(filename_varmap):
            os.remove(filename_varmap)
        else:
          # Remember that we successfully preprocessed this file already
          preprocessed_file_already_generated = True
    else:
      # Set filename to solve to the original cnf filename
      filename_to_solve = filename_cnf    
    
    # If there is a positive runtime, execute solver with given time limit
    if runtime != "-1":
      print "c Running " + solver + " for " + runtime + " seconds on " + filename_to_solve
      command = prog_timerun + ' ' + runtime + ' ' + prog_solvers[solver] + pre_instance_param_solvers[solver] + filename_to_solve + param_solvers[solver]
    else:
      print "c Running " + solver + " WITHOUT TIME LIMIT on " + filename_to_solve
      command = prog_solvers[solver] + pre_instance_param_solvers[solver] + filename_to_solve + param_solvers[solver]

    print "c    " + command    
    # Make sure that we don't use a positive preprocessing flag when Satelite has failed
    if satelite_failed == True:
      usepre = False
    # Execute solver command
    success = run_command_and_return_result(command, solver, usepre, filename_minisat_compliant_results, filename_cnf, filename_varmap)
    # If we found a solution break out of the loop 
    if success == True:
      break
        
  if success == False:
    # We should never get here, as the default solver was supposed to
    # be run without a timeout... well, unless it crashed
    print "c No solver, including the default solver " + solver_default + " without timelimit, could find a solution"
    print "s UNKNOWN"

  if not nofilecleanup:
    if os.path.exists(filename_cnf_pre):
      os.remove(filename_cnf_pre)
    if os.path.exists(filename_varmap):
      os.remove(filename_varmap)

  printTimer()
  # done!
  return 0

  
############################################################
# Execute given solver command and options and return result
############################################################
def run_command_and_return_result(command, solver, usepre, filename_minisat_compliant_results, filename_cnf, var_map_file):
  global solution_line_length
  success = False
  solverIn, solverOut, solverErr = os.popen3(command)
  solver_result = solverOut.read().rstrip()
  solver_status, solver_solution = parseSolverResult(solver, solver_result, usepre, filename_minisat_compliant_results)
  printTimer()
  if solver_status != "UNKNOWN":
    # instance solved!
    print "c Instance solved!"
    success = True
    if solver_status == "SATISFIABLE" and usepre:
      # first convert preprocessed lits to original lits        
      solver_solution = convertPreprocessedLitsToOrigLits(solver_solution, var_map_file)
      # now append these original solution literals as unit clauses to the
      # original formula, and then run the default solver on the extended file
      # to obtain full solution
      solver_solution = convertPartialSolutionToFull(filename_cnf, solver_solution)
      printTimer()
    print "s " + solver_status
    if solver_status == "SATISFIABLE":
      sys.stdout.write('v')
      i = 0
      for lit in solver_solution:
        sys.stdout.write(' ' + str(lit))
        i += 1
        if i % solution_line_length == 0:
          sys.stdout.write("\nv")
      sys.stdout.write("\n")      
  # Close all streams properly
  solverIn.close()
  solverOut.close()
  solverErr.close()
  # Return result
  return success

 
################################
##  parse command line arguments
################################
def parse_command_line():
  global tmpdir, timelimit, seed, nofilecleanup, testsolvers, verbosity

  try:
    opts, args = getopt.getopt(sys.argv[1:], "d:ht:s:v", ["checkall", "tmpdir=", "help", "timelimit=", "seed=", "nofilecleanup", "testsolvers", "verbose"])
  except getopt.GetoptError, err:
    print str(err)
    print usage
    sys.exit(1)

  do_checkall = False
  for option, value in opts:
    if option in ("--checkall"):
      do_checkall = True
    elif option in ("-d", "--tmpdir"):
      tmpdir = value
    elif option in ("-h", "--help"):
      print usage
      sys.exit(0)
    elif option in ("-t", "--timelimit"):
      timelimit = int(value)
      if timelimit <= 0:
        print "c ERROR: timelimit must be a positive integer"
        sys.exit(1)
    elif option in ("-s", "--seed"):
      seed = value
    elif option in ("--nofilecleanup"):
      nofilecleanup = True
    elif option in ("--testsolvers"):
      testsolvers = True
    elif option in ("-v", "--verbose"):
      verbosity = 1
    else:
      print "c ERROR: unrecognized option"
      print usage
      sys.exit(1)

  if do_checkall:
    checkAllFilesFoldersAvailable()
    sys.exit(0)

  if len(args) != 1:
    print usage
    sys.exit(1)
  else:
    instance_name = args[0]
    if not os.path.isfile(instance_name):
      print "c ERROR: cannot open CNF file " + instance_name
      print usage
      sys.exit(1)

  print "c    tmpdir         " + tmpdir
  print "c    timelimit      " + str(timelimit)
  print "c    seed           " + str(seed)
  print "c    verbosity      " + str(verbosity)

  return instance_name


#############################################
## check to make sure all necessary files
## and folders are present!
#############################################
def checkAllFilesFoldersAvailable():
  global prog_timerun, prog_features, prog_portfolio, prog_preprocessor, prog_solvers, datafolder_portfolio, file_features_test

  success = True

  print "c Making sure all necessary files and folders are present..."
  all_progs = [prog_timerun, prog_features, prog_portfolio, prog_preprocessor]
  all_progs.extend(prog_solvers.values())
  # use a little hack to get unique elements -- convert to set then back to list
  all_progs = list(set(all_progs))
  for prog in sorted(all_progs, key=str.lower):
    sys.stdout.write(("c   Checking for " + prog + ' ').ljust(55))
    if os.path.isfile(prog) and os.access(prog, os.X_OK):
      print "OK"
    else:
      print "NOT EXECUTABLE"
      success = False
  sys.stdout.write(("c   Checking for folder " + datafolder_portfolio + ' ').ljust(55))
  if os.path.isdir(datafolder_portfolio):
    print "OK"
  else:
    print "COACH's DATA FOLDER NOT FOUND"
    success = False
  sys.stdout.write(("c   Checking for file " + file_features_test + ' ').ljust(55))
  if os.path.isfile(file_features_test):
    print "OK"
  else:
    print "FEATURES TEST FILE NOT FOUND"
    success = False

  if not success:
    print "c Check failed. Please make sure all necessary files are available."
    print "c Exiting"
    sys.exit(1)

  print "c Check successful"


########################################
## print timer time
########################################
def printTimer():
  global timer_start, timer_prev
  timer_current = time.time()
  print "c TIMER: " + str(round(timer_current - timer_start, 2)) + " sec total, " + str(round(timer_current - timer_prev, 2)) + " sec since previous report"
  timer_prev = timer_current


########################################
## parse output of a solver
########################################
def parseSolverResult(_solver_name, _solver_result, _usepre, _filename_minisat_compliant_results):
  solver_status   = "UNKNOWN"
  solver_solution = []

  if _solver_name in compliant_solvers:
    lines = _solver_result.split('\n')
    for line in lines:
      words = line.split()
      if len(words) == 0:
        continue
      if words[0] == "s":
        solver_status = words[1]
      elif words[0] == "v":
        solver_solution.extend(words[1:])
      else:
        continue
  elif _solver_name in minisat_compliant_solvers:
    result_stream = open(_filename_minisat_compliant_results, 'r')
    line = result_stream.readline().rstrip()
    if line == "UNSAT":
      solver_status = "UNSATISFIABLE"
    elif line == "SAT":
      solver_status = "SATISFIABLE"
      for line in result_stream:
        solver_solution.extend(line.split())
    else:
      print "c  UNRECOGNIZED STATUS IN " + _filename_minisat_compliant_results
    result_stream.close()
    if not nofilecleanup:
      os.remove(_filename_minisat_compliant_results)
  else:
    print "UNCOMPLIANT SOLVER not yet supported, sorry!"

  return solver_status, solver_solution


#############################################
# convert solution in preprocessed variables
# to (partial) solution in original variables
#############################################
def convertPreprocessedLitsToOrigLits(_solver_solution, _filename_varmap):
  print "c Converting preprocessed instance solution back to original instance solution"
  # read varmap from file
  varmap_file = open(_filename_varmap, 'r')
  line = varmap_file.readline()
  line = varmap_file.readline()
  line = varmap_file.readline()
  varmap_file.close()
  # create variable mapping
  words = line.split()
  varmap = {0:0}
  newvar = 0
  for word in words:
    if word == '0':
      break
    newvar += 1
    varmap[newvar] = word
  # transform preprocessed literals to original literals
  mapped_solution = []
  for lit in _solver_solution:
    lit = int(lit)
    if lit >= 0:
      origlit = varmap[lit]
    else:
      origlit = '-' + varmap[-lit]
    mapped_solution.append(origlit)
  return mapped_solution


#######################################
# to get full original solution, append these literals from
# solution to the preprocessed instance as unit clauses to the
# original formula, and then run a solver on the extended file
#######################################
def convertPartialSolutionToFull(_filename_cnf, _partial_soln):
  global tmpdir
  partial_soln = _partial_soln[0:len(_partial_soln)-1]   # discard last entry, which is always 0
  filename_cnf_extended = tmpdir + '/' + os.path.basename(_filename_cnf) + '.extended'
  print "c    Writing unit-clause-extended CNF file to " + filename_cnf_extended
  cnf_extended_stream = open(filename_cnf_extended, 'w')
  cnf_extended_stream.write("c file " + _filename_cnf + " extended with partial solution\n")
  # parse and write out original CNF file but modify number of clauses
  num_clauses_orig = 0
  cnf_stream = open(_filename_cnf, 'r')
  for line in cnf_stream:
    words = line.split()
    if words[0] == 'p':
      num_vars = words[2]
      num_clauses = words[3]
      cnf_extended_stream.write("p cnf " + str(num_vars) + ' ' + str(int(num_clauses) + len(partial_soln)) + '\n')
    else:
      cnf_extended_stream.write(line)
  cnf_stream.close()
  for lit in partial_soln:
    cnf_extended_stream.write(lit + " 0\n")
  cnf_extended_stream.close()

  # now run a solver on this extended instance!
  print "c    Running " + solver_default + " on " + filename_cnf_extended
  command = prog_solvers[solver_default] + pre_instance_param_solvers[solver_default] + filename_cnf_extended + param_solvers[solver_default]
  print "c       " + command
  solverIn, solverOut, solverErr = os.popen3(command)
  solver_result = solverOut.read().rstrip()
  solver_status, solver_solution = parseSolverResult(solver_default, solver_result, False, '')
  assert solver_status == "SATISFIABLE"
  if not nofilecleanup:
    os.remove(filename_cnf_extended)
  return solver_solution


#######################################
if __name__ == "__main__":
  main()
#######################################


